:- consult(grilla), consult(env_primitives), consult(extras).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Predicados dinmicos empleados para representar el
% ESTADO INTERNO del ENTORNO.


:- dynamic posAg/3.             % Mantiene la posicin actual del agente.

:- dynamic ultAccAg/2.       % Mantiene la ltima accin realizada por
                                % el agente.

:- dynamic sensorChoque/2.   % Mantiene el valor actual obtenido por el sensor
                             % de choque.






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ag_registration_setup(+AgName)
%%
%% Initializes agent info

ag_registration_setup(AgName):-
                 assert(posAg(AgName, 2,3)),
                 assert(ultAccAg(AgName, none)),
                 assert(sensorChoque(AgName, 0)).
          




%% The Environment

start:- start_env,
        run.

%run:- termination.  % Si se verifica la condicin de terminacin,
%                    % finalizar la simulacin.


run:-
      dynamic_env_update,
      
      display_env, nl,          % para mostrar grilla por pantalla
      
      give_requested_percs,     % brinda percepcin a cada agente que la solicit
      
      sleep(1),                 % Time to Think: 1''
      
      excecute_available_actions,
      
      run.                       % cicla.


dynamic_env_update.

      
give_requested_percs:-
                   registered_agents(Ags),
                   forall((member(Ag, Ags), perc_request_available(Ag)), (generate_perc(Ag, Perc), give_percept(Perc, Ag))).
                   



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% generate_perc(+Ag, -Perc)
%
% calcula la prxima percepcin que recibir el agente Ag, donde Perc es de la
% forma [Choque, Suelo]

generate_perc(Ag, [Choque, Suelo]):-
                  sensorChoque(Ag, Choque),        % se obtiene el valor actual del sensor de choque.
                  posAg(Ag, FAct, CAct),       % se obtiene posicin actual del agente.
                  c(FAct, CAct, Suelo).        % se obtiene el valor actual del sensor de suelo.






excecute_available_actions:-
                            registered_agents(Ags),
                            forall((member(Agent, Ags), action_available(Action, Agent)), update_env_state(Agent, Action)).
                            % Para todos los agentes Ag tales que "solicitaron" ejecutar una accin Action, actualizar el estado
                            % actual del entorno para reflejar la ejecucin de Action por parte de Agent.


                            
% update_env_state(+Agent, +Action)

update_env_state(Agent, tomar):- retract(ultAccAg(Agent, _)),
                                 assert(ultAccAg(Agent, tomar)),
                                 
                                 posAg(Agent, FAct, CAct),                                 % se obtiene posicin actual del agente
                                 
                                 retract(c(FAct, CAct, _Suelo)),
                                 assert(c(FAct, CAct, -)),                             % se refleja que ya no hay nada en el suelo
                                 
                                 retract(sensorChoque(Agent, _)),
                                 assert(sensorChoque(Agent, 0)).                              % el sensor de choque queda en 0
                                 


update_env_state(Agent, Dir):- posAg(Agent, FAct,CAct),                % se obtiene posicin actual del agente

                   pos_destino(Dir, FAct, CAct, FDest, CDest),       % calcula la posicin a la que
                                                                     % intenta moverse el agente.

                   not(c(FDest, CDest, *)),!,            % Si en la posicin destino NO hay pared...

                   retract(posAg(Agent, FAct,CAct)),   % actualiza la posicin actual del agente
                   assert(posAg(Agent, FDest,CDest)),  % para reflejar que se movi.


                   retract(ultAccAg(Agent, _A)),
                   assert(ultAccAg(Agent, Dir)),       % actualiza la ltima accin del agente


                   retract(sensorChoque(Agent, _)),    % actualiza el valor del sensor de choque
                   assert(sensorChoque(Agent, 0)).     % para reflejar que NO choc.


update_env_state(Agent, Dir):-
                   retract(ultAccAg(Agent, _A)),
                   assert(ultAccAg(Agent, Dir)),     % actualiza la ltima accin del agente
                   
                   retract(sensorChoque(Agent, _)),          % actualiza el valor del sensor de choque
                   assert(sensorChoque(Agent, 1)).          % para reflejar que choc.



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Auxiliares de update_env_state/2 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% pos_destino(+Dir, +FAct, +CAct, -FDest, -CDest)
%
% Dada una direccin Dir (norte, sur, este u oeste) y una posicin (FAct, CAct)
% determina la posicin (FDest, CDest) adyacente a (FAct, CAct) en la direccin
% Dir.


pos_destino(norte, FAct, CAct, Fnorte, CAct ):-
                   next(Fnorte, FAct).

pos_destino(sur, FAct, CAct, Fsur, CAct ):-
                   next(FAct, Fsur).

pos_destino(este, FAct, CAct, FAct, Ceste ):-
                   next(CAct, Ceste).

pos_destino(oeste, FAct, CAct, FAct, Coeste ):-
                   next(Coeste, CAct).