%% Agent-Environment interaction
%%
%% Environment primitives (for interaction with the agent)

:- [interface].

:- dynamic id_name/2.


% start_env
%
% Inicia la ejecucin del entorno.
% Crea un thread para "escuchar" y resolver pedidos de registracin
% por parte de agentes.

start_env:- start_server(8001),
            write('antes de crear el thread'), nl,
            thread_create(registration_handler,_,[]).
            
            

registration_handler:- write('entr a registration_handler/1'), nl,
                      recv_msg(AgID, i_am(AgName)),
                      write('recibi mensaje i_am de '), write(AgName), nl,
                      (id_name(_OtroAgID, AgName), % El nombre ya est siendo usado por otro agente.
                       send_msg(AgID, used_name)
                        ;
                       assert(id_name(AgID, AgName)), % Se registra el nuevo nombre
                       %ip:dont_fail(ag_registration_setup(AgName)),
                       dont_fail(ag_registration_setup(AgName)),
                       send_msg(AgID, connected)
                       ),
                       sleep(3),
                       registration_handler.


% registered_agents(-Agents)
%
% Retorna una lista con todos los (nombres de) agentes registrados al momento.

registered_agents(Agents):- findall(Agent, id_name(_ID, Agent), Agents).


% perc_request_available(-Ag)
%
% Retorna el nombre de un agente que solicit una percepcin, si existe.
% En caso de no existir tal agente, falla.
%
% gets percept request from Ag if available

perc_request_available(Ag):-
                     id_name(AgID, Ag),
                     recv_msg(AgID, perc_req, 0.01). %antes 0.001
                     

% give_percept(+Perc, +Ag)
%
% brinda la percepcin Perc al agente Ag.

give_percept(Perc, Ag):-
                     id_name(AgID, Ag),
                     send_msg(AgID, percept(Perc)). % la estructura percept va por si acaso. No estoy seguro que la necesite
                     
                     
% action_available(-Action, -Ag)
%
% Retorna el nombre de un agente que solicit efectuar una accin, si existe,
% junto con la accin correspondiente.
% En caso de no existir tal agente, falla.

                    
action_available(Action, Ag):-
                     id_name(AgID, Ag),
                     recv_msg(AgID, action(Action), 0.01),  % la estructura action es fundamental, sino el get_action puede consumir
                     send_msg(AgID, ack).                   % un pedido de percepcin (de un agente que entr tarde por ejemplo, o simplemente
                                                            % se tard en pedir la percepcin).


%%%%%%%%%%%%%%% AUXILIARES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

dont_fail(Goal):- call(Goal), !.

dont_fail(_Goal).